﻿Public Class CalculatorForm
    ' I will keep a record to the textbox that should receive the input value.
    ' Each time the user enters one of the value textboxes, this variable
    ' will be updated.
    Private receivingTextBox As TextBox = value1TextBox
    Private currentBase As BaseKey


    Private Enum BaseKey As Integer
        Dec = 10
        Bin = 2
        Hex = 16
    End Enum


#Region "Methods"
    ' Sets initial state of controls.
    Private Sub InitializeForm()
        ' Indicate decimal base as the initial state.
        decRadioButton.Checked = True

        ' Clear the textboxes and operator label.
        value1TextBox.Clear()
        value2TextBox.Clear()
        resultLabel.Text = ""
        operatorLabel.Text = ""

        ' Disable the value2 textbox.
        value2TextBox.Enabled = False
    End Sub

    ' Enables keys based on selected base.
    Private Sub EnableState(ByVal base As BaseKey)
        Select Case base
            Case BaseKey.Dec
                ToggleDecimalState(True)
                ToggleHexadecimalState(False)

            Case BaseKey.Bin
                ToggleDecimalState(False)
                ToggleBinaryState(True)
                ToggleHexadecimalState(False)

            Case BaseKey.Hex
                ToggleDecimalState(True)
                ToggleHexadecimalState(True)
        End Select
    End Sub ' ToggleEnableState

    ' Enables/disables the decimal keys.
    Private Sub ToggleDecimalState(ByVal enableState As Boolean)
        zeroButton.Enabled = enableState
        oneButton.Enabled = enableState
        twoButton.Enabled = enableState
        threeButton.Enabled = enableState
        fourButton.Enabled = enableState
        fiveButton.Enabled = enableState
        sixButton.Enabled = enableState
        sevenButton.Enabled = enableState
        eightButton.Enabled = enableState
        nineButton.Enabled = enableState
    End Sub

    ' Enables/disables the binary keys.
    Private Sub ToggleBinaryState(ByVal enableState As Boolean)
        zeroButton.Enabled = enableState
        oneButton.Enabled = enableState
    End Sub

    ' Enables/disables the hexadecimal keys.
    Private Sub ToggleHexadecimalState(ByVal enableState As Boolean)
        AButton.Enabled = enableState
        BButton.Enabled = enableState
        CButton.Enabled = enableState
        DButton.Enabled = enableState
        EButton.Enabled = enableState
        FButton.Enabled = enableState
    End Sub

    ' Performs the desired operation
    Private Function PerformOperation(ByVal val1 As String, ByVal val2 As String, ByVal op As String) As String
        ' Convert the string representation of a number in 
        ' a given base to an integer.
        Dim value1 As Int32 = Convert.ToInt32(val1, currentBase)
        Dim value2 As Int32 = Convert.ToInt32(val2, currentBase)
        Dim result As Int32
        Dim resultAsString As String = ""

        Select Case op
            Case "+"
                result = value1 + value2

            Case "-"
                result = value1 - value2
        End Select

        ' Convert the integer result to a string representation in the
        ' specified base.
        resultAsString = Convert.ToString(result, currentBase).ToUpper

        Return resultAsString
    End Function

    ' Translates values to specified base.
    Private Sub UpdateValues(ByVal fromBase As BaseKey, ByVal toBase As BaseKey)
        If value1TextBox.TextLength > 0 Then
            value1TextBox.Text = Convert.ToString(Convert.ToInt32(value1TextBox.Text, fromBase), toBase).ToUpper()
        End If

        If value2TextBox.TextLength > 0 Then
            value2TextBox.Text = Convert.ToString(Convert.ToInt32(value2TextBox.Text, fromBase), toBase).ToUpper()
        End If

        If resultLabel.Text.Length > 0 Then
            resultLabel.Text = Convert.ToString(Convert.ToInt32(resultLabel.Text, fromBase), toBase).ToUpper()
        End If
    End Sub
#End Region



#Region "Event Handlers"

    ' Initializes the form by setting the base state and setting the focus.
    Private Sub CalculatorForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        InitializeForm()
    End Sub

    ' Handles all the radio buttons at once.
    ' Enables the keys's enable state based on the selected radio button. It 
    ' also converts the values to the appropriate base.
    Private Sub allRadioButton_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles hexRadioButton.CheckedChanged, decRadioButton.CheckedChanged, binRadioButton.CheckedChanged
        ' Identify the radio button whose state has changed.
        Dim rb As RadioButton = CType(sender, RadioButton)

        ' The first time this event fires is when one radio button looses the
        ' check. The second time is when the new radio button receives it.
        If rb.Checked = True Then
            Dim fromBase As BaseKey = currentBase

            ' If both values are given then proceed, else skip.

            Select Case rb.Name
                Case "decRadioButton"
                    currentBase = BaseKey.Dec

                Case "binRadioButton"
                    currentBase = BaseKey.Bin

                Case "hexRadioButton"
                    currentBase = BaseKey.Hex
            End Select

            UpdateValues(fromBase, currentBase)

            ' Each time a radio button is checked two things happen: 1) one radio
            ' button is unselected and 2) one is selected. Since one is always
            ' selected, we update the keys to reflect this new selection.
            EnableState(currentBase)
        End If
    End Sub

    ' Handles both operators at once.
    ' Enables the second value textbox.
    Private Sub operatorButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles plusButton.Click, minusButton.Click
        ' Update the operator label to reflect the operator key that was clicked.
        operatorLabel.Text = CType(sender, Button).Text

        ' Enable the value2 textbox.
        value2TextBox.Enabled = True
        value2TextBox.Focus()
    End Sub

    ' Initializes the form and sets the focus to the first value.
    Private Sub clearButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles clearButton.Click
        ' Initialize the form.
        InitializeForm()

        ' Give the focus to the first value field.
        value1TextBox.Focus()
    End Sub

    ' Appends the text value of the clicked button to the active textbox.
    Private Sub keyButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles zeroButton.Click, twoButton.Click, threeButton.Click, sixButton.Click, sevenButton.Click, oneButton.Click, nineButton.Click, fourButton.Click, fiveButton.Click, FButton.Click, eightButton.Click, EButton.Click, DButton.Click, CButton.Click, BButton.Click, AButton.Click
        ' Append the text value of the clicked key to the textbox that has the focus.
        receivingTextBox.AppendText(CType(sender, Button).Text)
    End Sub

    ' Occurs each time the contol becomes the active one.
    Private Sub valueTextBox_Enter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles value2TextBox.Enter, value1TextBox.Enter
        receivingTextBox = CType(sender, TextBox)
    End Sub


    ' Performs the desired computation and updates the result textbox.
    Private Sub equalButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles equalButton.Click
        Select Case currentBase
            Case BaseKey.Bin
                With value1TextBox
                    .Text = .Text.PadLeft(32, .Text.Chars(0))
                End With

                With value2TextBox
                    .Text = .Text.PadLeft(32, .Text.Chars(0))
                End With

            Case BaseKey.Hex
                With value1TextBox
                    If Text.Chars(0) < "0"c OrElse Text.Chars(0) > "7"c Then
                        .Text = .Text.PadLeft(8, "F"c).ToUpper
                    Else
                        .Text = .Text.PadLeft(8, .Text.Chars(0)).ToUpper
                    End If
                End With

                With value2TextBox
                    If Text.Chars(0) < "0"c OrElse Text.Chars(0) > "7"c Then
                        .Text = .Text.PadLeft(8, "F"c).ToUpper
                    Else
                        .Text = .Text.PadLeft(8, .Text.Chars(0)).ToUpper
                    End If
                End With
        End Select
        resultLabel.Text = PerformOperation(value1TextBox.Text, value2TextBox.Text, operatorLabel.Text)
    End Sub

#End Region
End Class ' CalculatorForm
