﻿Imports System.IO


Public Class CustomerList
    Private Structure Customer
        Public First As String
        Public Last As String
        Public City As String
        Public State As String
    End Structure

    Private customers(99) As Customer


    Private Function OpenFile(ByVal fileName As String) As StreamReader
        Dim sr As StreamReader

        Try
            sr = New StreamReader(fileName)
        Catch ex As IOException
            Throw New IOException("Error opening " & fileName)
        End Try
        Return sr
    End Function

    Private Sub CloseFile(ByVal sr As StreamReader)

        If sr IsNot Nothing Then
            Try
                sr.Close()
            Catch ex As IOException
                MessageBox.Show("Error closing file.",
                                "File Error",
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Error)
            End Try
        End If
    End Sub

    Private Sub InitializeArrayFromFile(ByVal sr As StreamReader)
        Dim record As String
        Dim fields() As String
        Dim index As Int32

        Try
            Do While Not sr.EndOfStream
                record = sr.ReadLine()
                fields = record.Split(","c)

                With customers(index)
                    .First = fields(0)
                    .Last = fields(1)
                    .City = fields(2)
                    .State = fields(3)
                End With
                index += 1
            Loop

            ' Redim the array to conserve memory.
            ReDim Preserve customers(index - 1)

        Catch ex As IOException
            MessageBox.Show("Error reading file",
                            "File Error",
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error)
        Finally
            ' Close the filename, since we do not need it anymore.
            CloseFile(sr)
        End Try
    End Sub



    Private Sub SortArray()
        ' Sorts the array based on user selections.
        Dim swapIndex As Int32

        ' What field should be sort on?
        For topIndex = 0 To customers.GetUpperBound(0) - 1
            swapIndex = topIndex

            For index = topIndex + 1 To customers.GetUpperBound(0)
                If ascendingRadioButton.Checked = True Then
                    ' Ascending order.
                    If lastRadioButton.Checked = True Then
                        ' by last.
                        If customers(index).Last < customers(swapIndex).Last Then swapIndex = index
                    ElseIf firstRadioButton.Checked = True Then
                        ' by first.
                        If customers(index).First < customers(swapIndex).First Then swapIndex = index
                    ElseIf cityRadioButton.Checked = True Then
                        ' by city.
                        If customers(index).City < customers(swapIndex).City Then swapIndex = index
                    Else
                        ' by state
                        If customers(index).State < customers(swapIndex).State Then swapIndex = index
                    End If
                Else
                    ' Descending order.
                    If lastRadioButton.Checked = True Then
                        ' by last.
                        If customers(index).Last > customers(swapIndex).Last Then swapIndex = index
                    ElseIf firstRadioButton.Checked = True Then
                        ' by first.
                        If customers(index).First > customers(swapIndex).First Then swapIndex = index
                    ElseIf cityRadioButton.Checked = True Then
                        ' by city.
                        If customers(index).City > customers(swapIndex).City Then swapIndex = index
                    Else
                        ' by state
                        If customers(index).State > customers(swapIndex).State Then swapIndex = index
                    End If
                End If
            Next index

            ' Swap the two elements.
            Dim temp As Customer = customers(topIndex)
            customers(topIndex) = customers(swapIndex)
            customers(swapIndex) = temp
        Next topIndex
    End Sub



    Private Sub LoadListBoxWithArray()
        Dim formatString As String = "{0,-10}{1,-10}{2,-20}{3,-15}"

        ' Sort the array before we continue on.
        SortArray()

        ' Add a header to the list first.
        With customerDisplayListBox.Items
            .Clear()
            .Add(String.Format(formatString, "First", "Last",
                                              "City", "State"))
            .Add("--------------------------------------------------------")
        End With

        ' Add remaining records.
        For Each Customer In customers
            With customerDisplayListBox.Items
                .Add(String.Format(formatString,
                                   Customer.First,
                                   Customer.Last,
                                   Customer.City,
                                   Customer.State))
            End With
        Next
    End Sub


    Private Sub ExitToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ExitToolStripMenuItem.Click
        ' Exit the application by closing the form.
        Me.Close()
    End Sub

    Private Sub OpenToolStripMenuItem_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles OpenToolStripMenuItem.Click
        ' Present the user with the openFileDialog so they may choose the file to open.
        Dim dialogResult As DialogResult
        Dim fileName As String

        With OpenFileDialog
            .InitialDirectory = Application.StartupPath
            dialogResult = .ShowDialog()
            fileName = .FileName
        End With

        ' Verify they did not cancel out.
        If dialogResult <> dialogResult.Cancel Then
            Try
                Dim sr As StreamReader

                ' Open the file.
                sr = OpenFile(fileName)

                ' Now that the file has been specified, open the file and populate the array.
                InitializeArrayFromFile(sr)

                ' Enable the customerlist groupbox.
                customerListGroupBox.Enabled = True

                ' The array is populated, so load the listbox.
                LoadListBoxWithArray()

            Catch ex As IOException
                MessageBox.Show(ex.Message,
                                "File Error",
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Error)
            End Try
        End If
    End Sub

    Private Sub allRadioButton_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lastRadioButton.CheckedChanged, stateRadioButton.Click, firstRadioButton.Click, cityRadioButton.Click, descendingRadioButton.CheckedChanged, ascendingRadioButton.CheckedChanged
        ' Reload the listbox to reflect new sorting order, based on the selected radiobutton.
        LoadListBoxWithArray()
        MessageBox.Show("Just a test",
                        "Testing Dialog",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error)
    End Sub
End Class
