﻿' Abstract superclass
Public MustInherit Class Student
    Public Property FirstName() As String
    Public Property LastName() As String
    Public Property AcademicMajor() As String


    Public Sub New(ByVal first As String, ByVal last As String,
                   ByVal major As String)
        FirstName = first
        LastName = last
        AcademicMajor = major
    End Sub

    Public Overrides Function ToString() As String
        Return String.Format("{0} {1}{2}{3}", FirstName, LastName,
                             vbCrLf, "Major: ", AcademicMajor)
    End Function
End Class



' Concrete subclass
Public Class Undergraduate
    Inherits Student
    Public Property AcademicMinor() As String

    Private _year As String
    Public Property AcademicYear() As String
        Get
            Return _year
        End Get
        Set(ByVal value As String)
            Select Case value
                Case "Freshman", "Sophomore", "Junior", "Senior"
                    _year = value
                Case Else
                    Throw New ArgumentOutOfRangeException("Academic year " &
                    "must be either Freshman, Sophomore, Junior or Senior")
            End Select
        End Set
    End Property


    Public Sub New(ByVal first As String, ByVal last As String,
                   ByVal major As String, ByVal minor As String,
                   ByVal year As String)
        MyBase.New(first, last, major)
        AcademicMinor = minor
        AcademicYear = year
    End Sub

    Public Overrides Function ToString() As String
        Return MyBase.ToString() &
               String.Format("{0}Minor: {1}{0}Year: {2}",
                             vbCrLf, AcademicMinor, AcademicYear)
    End Function
End Class


' Concrete subclass
Public Class Graduate
    Inherits Student


    Private _thesisOption As Boolean
    Public Property ThesisOption() As Boolean
        Get
            Return _thesisOption
        End Get
        Set(ByVal value As Boolean)
            Select Case value
                Case True, False
                    _thesisOption = value
                Case Else
                    Throw New ArgumentOutOfRangeException("Thesis option " &
                                "must be true or false")
            End Select
        End Set
    End Property

    Public Sub New(ByVal first As String, ByVal last As String,
                   ByVal major As String, ByVal thesis As Boolean)
        MyBase.New(first, last, major)
        ThesisOption = thesis
    End Sub

    Public Overrides Function ToString() As String
        Return MyBase.ToString() &
               String.Format("{0}Thesis: {1}", vbCrLf, ThesisOption.ToString())
    End Function
End Class
