﻿Imports System.IO

Public Class StateTutorForm
    Private imageFolderPath As String = Application.StartupPath & "\..\Images"
    Private stateInfoFilePath As String = Application.StartupPath & "\.."
    Private currentStateIndex As Int32

    ' Notice the New in this declaration. This allocates the object itself,
    ' thus we currently have an array of 50 references to USState objects.
    Private states As New USStates


    ' Name: LoadStatesFromFile()
    ' Opens the input file and loads the state information into the object.
    ' The input file is in alphabetical order, so no need to adjust the object,
    ' i.e. sort it.
    Private Sub LoadStatesFromFile(ByVal filename As String)
        Dim sr As StreamReader = Nothing
        Try
            ' Ok, let's attempt to open the file and load the data.
            sr = New StreamReader(stateInfoFilePath & "\" & filename)

            LoadStatesFromStream(sr)
        Catch ex As IOException
            MessageBox.Show(ex.Message,
                            "File IO Error",
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error)
        Finally
            ' The finally block will execute every time, both on success
            ' and failure. This is the perfect place to close the file.
            If sr IsNot Nothing Then
                ' File is open, so let's close it.
                Try
                    sr.Close()
                Catch ex As IOException
                    MessageBox.Show(ex.Message,
                                    "File IO Error",
                                    MessageBoxButtons.OK,
                                    MessageBoxIcon.Error)
                End Try
            End If
        End Try
    End Sub

    ' Name: LoadStatesFromStream()
    ' Performs the actual reading from the input stream.
    Private Sub LoadStatesFromStream(ByVal sr As StreamReader)
        Try
            Dim stateIndex As Int32 = 0
            Do While Not sr.EndOfStream
                Dim stateRecord As String
                Dim stateFields() As String

                ' Read one record at a time and then split it up into tokens.
                stateRecord = sr.ReadLine
                stateFields = stateRecord.Split(","c)

                ' Load the states object from the stateFields array.
                states.Items(stateIndex) = New USState(stateFields(0),
                                                       stateFields(1),
                                                       stateFields(2),
                                                       stateFields(3))
                stateIndex += 1
            Loop
        Catch ex As IOException
            ' We will detect the exception, but let the caller handle it.
            ' You are welcome! :)
            Throw ex
        End Try

    End Sub


    ' Name: InitializeForm()
    ' Loads the first state's image and flag and updates the state, capitol
    ' labels.
    Private Sub InitializeForm()
        currentStateIndex = 0
        UpdateStateInfo()
    End Sub

    ' Name: UpdateStateInfo()
    ' Updates the controls with the state information.
    ' It cycles through all the states when it reaches a boundary.
    Private Sub UpdateStateInfo()
        ' The currentStateIndex will cycle when it reaches one of the
        ' boundaries.
        currentStateIndex = (USStates.NumberOfStates + currentStateIndex) Mod
                             USStates.NumberOfStates

        With states.Items(currentStateIndex)
            stateLabel.Text = .Name
            capitalLabel.Text = .Capital
            statePictureBox.Image = Image.FromFile(imageFolderPath & "\" & .StateImageFilename)
            flagPictureBox.Image = Image.FromFile(imageFolderPath & "\" & .FlagImageFilename)
        End With
    End Sub

    ' Loads the state information and initializes the form.
    Private Sub StateTutorForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' Load the state information into our states object.
        LoadStatesFromFile("StatesTutor.txt")

        ' Now, initialize the form and let's get started.
        InitializeForm()
    End Sub

    ' Displays the next or previous state information.
    Private Sub navigationButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles previousButton.Click, nextButton.Click
        Dim btn As Button = CType(sender, Button)

        ' Let's determine if we are moving to the next or previous state. This
        ' will help decide how we are going to change the currentStateIndex.
        Select Case btn.Text
            Case ">>"
                currentStateIndex += 1
            Case "<<"
                currentStateIndex -= 1
        End Select

        ' Update the UI with the new state.
        UpdateStateInfo()
    End Sub
End Class
