﻿Imports System.IO

Public Class StatesExplorerForm
    Private imageFilesPath As String = Application.StartupPath & "\..\Images"
    Private stateInfo(49, 1) As String

    ' Name: LoadImageList()
    ' Loads the statesImageList with all the state images.
    Private Sub LoadImageList()

        ' I will use the option infer here to declare the object 
        ' stateImageFilePaths. This is a generic collection of strings.
        Dim stateImageFilePaths = Directory.EnumerateFiles(imageFilesPath, "state_*.gif")

        ' Let's cycle through each path and add each image to the imagelist.
        Dim filename As String
        Dim key As String

        For Each stateImageFilePath In stateImageFilePaths
            ' I will use the name of the state as the key, for easy retrieval
            ' later on. Since each image filename is prefixed with "state_",
            ' I will first remove this prefix.
            With stateImageFilePath
                filename = .Substring(.LastIndexOf("\") + 1)
            End With

            key = filename.Replace("state_", "")

            ' Ok, now I can add the name of the state as its key in the
            ' imageList.
            statesImageList.Images.Add(key, Image.FromFile(stateImageFilePath))
        Next

        ' Since we are finished with the collection, let's release it, to save
        ' on some resources.
        stateImageFilePaths = Nothing
    End Sub

    ' Name: LoadStateInfo()
    ' Opens the states info file and loads the statesInfo array.
    Private Sub LoadStateInfo()
        Dim sr As StreamReader = Nothing

        Try
            sr = New StreamReader(Application.StartupPath & "\..\StateCapitals.txt")

            Dim stateRecord As String
            Dim stateFields() As String
            Dim stateIndex As Int32 = 0

            Do While Not sr.EndOfStream
                stateRecord = sr.ReadLine
                stateFields = stateRecord.Split(","c)

                stateInfo(stateIndex, 0) = stateFields(0)
                stateInfo(stateIndex, 1) = stateFields(1)
                stateIndex += 1
            Loop
        Catch ex As IOException
            MessageBox.Show(ex.Message,
                            "File IO Error",
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error)
        Finally
            If sr IsNot Nothing Then
                Try
                    sr.Close()
                Catch ex As IOException
                    MessageBox.Show(ex.Message,
                                    "File IO Error",
                                    MessageBoxButtons.OK,
                                    MessageBoxIcon.Error)
                End Try
            End If
        End Try
    End Sub


    ' Name: AddTreeViewNodes()
    ' Adds a node for each unique state letter. This will be like an index to
    ' all the states that start with a given letter.
    Private Sub AddTreeViewNodes()
        ' Since the names of the states are in the imageList, I will cycle
        ' through this array and check the key value of each state.

        ' To insure unique entries, I will use a collection which has this
        ' functionality already built in. If a duplicate value is attempted
        ' to be added, an exception is thrown. I will catch it and ignore it.
        Dim indexKeys As New Collection

        For Each key In statesImageList.Images.Keys
            Try
                indexKeys.Add(key.Substring(0, 1).ToUpper, key.Substring(0, 1).ToUpper)
            Catch ex As ArgumentException
            End Try
        Next

        ' Now I can add the unique entries as nodes into the treeView.
        For Each indexKey In indexKeys
            With statesTreeView.Nodes
                .Add(indexKey.ToString)
            End With
        Next
    End Sub

    ' Name: InitializeListView()
    ' Initializes the ListView to show two columns, one for the state and one
    ' for the capital.
    Private Sub InitializeListView()
        With statesListView
            .Columns.Add("State")
            .Columns.Add("Capital")
            .View = View.Details
        End With
    End Sub

    Private Sub StatesExplorerForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        LoadImageList()
        LoadStateInfo()
        AddTreeViewNodes()
        InitializeListView()
    End Sub

    Private Sub statesTreeView_AfterSelect(ByVal sender As Object, ByVal e As System.Windows.Forms.TreeViewEventArgs) Handles statesTreeView.AfterSelect
        ' Populate the listview with the state's name and capital
        Dim node As TreeNode = CType(sender, Windows.Forms.TreeView).SelectedNode

        Dim stateIndex As Int32 = 0

        ' Before we populuat the listview, let's clear any old states displayed.
        statesListView.Items.Clear()

        ' First skip all the nodes that don't start with the selected letter.
        Do While Not stateInfo(stateIndex, 0).StartsWith(node.Text)
            stateIndex += 1
        Loop

        ' Now select all the states with the same first letter.
        'TODO: Check why it crashes when it attempts to select "W"
        Do While stateInfo(stateIndex, 0).StartsWith(node.Text) AndAlso stateIndex < stateInfo.GetUpperBound(0)
            Dim item As New ListViewItem(New String() {stateInfo(stateIndex, 0), stateInfo(stateIndex, 1)})
            statesListView.Items.Add(item)
            stateIndex += 1
        Loop

        ' Autoresize the colunns to fit the contents or the column header,
        ' whichever is wider.
        statesListView.AutoResizeColumns(ColumnHeaderAutoResizeStyle.ColumnContent)
        statesListView.AutoResizeColumns(ColumnHeaderAutoResizeStyle.HeaderSize)
    End Sub
End Class
